/*
 * Copyright E. Bigeon (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * Socket implementation of GCLC.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-socket:fr.bigeon.gclc.socket.ThreadedServerConsoleManager.java
 * Created on: Jun 1, 2016
 */
package fr.bigeon.gclc.socket;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import fr.bigeon.gclc.manager.ConsoleManager;
import fr.bigeon.smu.StringEncoder;

/** The console manager for socket communication
 *
 * @author Emmanuel Bigeon */
public class ThreadedServerConsoleManager implements ConsoleManager {

    /** The eol character */
    private static final String EOL = "\n"; //$NON-NLS-1$
    /** The class logger */
    private static final Logger LOGGER = Logger
            .getLogger(ThreadedServerConsoleManager.class.getName());
    /** The prompting sequence */
    private String prompt = new String();
    /** The buffer of data to send to the user */
    private StringBuilder buffer = new StringBuilder();
    /** The synchronized object */
    private final Object promptingLock;
    /** The output to write data comming from the application */
    private PrintWriter output;
    /** The encoder to encode data coming from the application */
    private final StringEncoder encoder;
    /** The input to wait data from the user */
    private BufferedReader input;
    /** the prompting status */
    private boolean doPrompt;
    /**
     * 
     */
    private boolean closed = false;

    /** Create the console manager.
     * 
     * @param encoder the encoder for output
     * @param promptingLock the synchronization object */
    public ThreadedServerConsoleManager(StringEncoder encoder,
            Object promptingLock) {
        super();
        this.encoder = encoder;
        this.promptingLock = promptingLock;
    }

    /** @param input the input to set */
    public void setInput(BufferedReader input) {
        this.input = input;
    }

    /** @param output the output to set */
    public void setOutput(PrintWriter output) {
        this.output = output;
    }

    @Override
    public void setPrompt(String prompt) {
        this.prompt = prompt;
    }

    @Override
    public String prompt(String message) {
        buffer.append(message);
        String userInput = new String();
        boolean prompting = true;
        while (prompting) {
            // Send buffer content
            output.println(encoder.encode(buffer.toString()));
            try {
                synchronized (promptingLock) {
                    doPrompt = true;
                    promptingLock.notify();
                }
                userInput = input.readLine();
                doPrompt = false;
                prompting = false;
            } catch (final IOException e) {
                LOGGER.log(Level.SEVERE, "input reading error", e); //$NON-NLS-1$
            }
        }
        // Renew buffer
        buffer = new StringBuilder();
        return userInput;
    }

    /** @return the prompting status */
    public synchronized boolean isPrompting() {
        return doPrompt;
    }

    @Override
    public String prompt() {
        return prompt(prompt);
    }

    @Override
    public void println(String message) {
        buffer.append(message + EOL);

    }

    @Override
    public void println() {
        buffer.append(EOL);
    }

    @Override
    public void print(String text) {
        buffer.append(text);
    }

    @Override
    public String getPrompt() {
        return prompt;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#close() */
    @Override
    public void close() throws IOException {
        // Do nothing
        this.closed = true;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#isClosed() */
    @Override
    public boolean isClosed() {
        return closed;
    }

}
